from twisted.trial.unittest import TestCase
from twisted.python.failure import Failure
from tada import Tadah

class MySQLTest(TestCase):
    """
    This TestCase has the following requirements:
        MySQL:
            a database named 'test' that this script can access without username/password
    """

    def testConnectFail(self):
        return Tadah("MySQLdb",db="tst").connect().addErrback(lambda _: True)

    def testConnectPass(self):
        return Tadah("MySQLdb",db="test").connect()

    def testGetCursorManually(self):
        def gotCursor(cursor):
            from MySQLdb.cursors import Cursor
            self.assertEquals(cursor.__tada_cursor__.__class__,Cursor)
            return True
        def gotConnection(connection):
            return connection.cursor().addCallback(gotCursor)
        return Tadah("MySQLdb",db="test").connect().addCallback(gotConnection)

    def testGetCursor(self):
        def gotCursor(cursor):
            from MySQLdb.cursors import Cursor
            self.assertEquals(cursor.__tada_cursor__.__class__,Cursor)
            return True
        return Tadah("MySQLdb",db="test").cursor().addCallback(gotCursor)

    def testDropNonExistantTable(self):
        def mustFailDeferredResult(result):
            assert(isinstance(result,Failure))
        return Tadah("MySQLdb",db="test").execute("DROP TABLE `thisTableShouldNotExist`").addBoth(mustFailDeferredResult)

    def testDropTableIf(self):
        def cursorContext(cursor):
            return True
        return Tadah("MySQLdb",db="test").execute("DROP TABLE IF EXISTS `thisTableShouldNotExist`").addCallback(cursorContext)

    def testCreateTable(self):
        from MySQLdb.cursors import Cursor
        def cursorContext(cursor):
            return True
        def tableCreated(cursor):
            return cursor.execute("DROP TABLE `thisTableCouldExist`").addCallback(cursorContext)
        def createTable(cursor):
            return cursor.execute("CREATE TABLE `thisTableCouldExist` ( id int )").addCallback(lambda _: tableCreated(cursor))
        return Tadah("MySQLdb",db="test").execute("DROP TABLE IF EXISTS `thisTableCouldExist`").addCallback(createTable)

