""" This set of tests, assumes that tada.Tada and tada.Tadah are fully functional"""

from twisted.trial.unittest import TestCase
from twisted.python.failure import Failure

from tada import Tada,Tadah,tao

class SimpleTao(tao.Table):
    __table__ = 'simple'
    num = tao.IntCol('num')

class SQLiteTest(TestCase):

    def test_CreateTableSQL(self):
        def created(cursor):
            from pysqlite2.dbapi2 import Cursor
            self.assertEquals(cursor.__tada_cursor__.__class__,Cursor)
        simple = SimpleTao(Tadah("pysqlite2.dbapi2",self.mktemp()))
        self.assertEqual(repr(simple),"""CREATE TABLE simple (
num int NOT NULL);""")

    def test_CreateTable(self):
        def created(result):
            self.assertEquals(result,True)
        simple = SimpleTao(Tadah("pysqlite2.dbapi2",self.mktemp()))
        return simple.create().addCallback(created)

    def test_DropNonExistingTable(self):
        def failed(failure):
            from pysqlite2.dbapi2 import OperationalError
            self.assertEquals(failure.value,OperationalError)
            self.assertEquals(str(failure.type),"no such table: simple")
        simple = SimpleTao(Tadah("pysqlite2.dbapi2",self.mktemp()))
        return simple.drop().addErrback(failed)

    def test_DropTable(self):
        simple = SimpleTao(Tadah("pysqlite2.dbapi2",self.mktemp()))
        def dropped(cursor):
            from pysqlite2.dbapi2 import Cursor
            self.assertEquals(cursor.__tada_cursor__.__class__,Cursor)
        def created(cursor):
            return simple.drop().addCallback(dropped)
        return simple.create().addCallback(created)

    def test_BasicSelect(self):
        filename = self.mktemp()
        dbapi = Tadah("pysqlite2.dbapi2",filename)
        simple = SimpleTao(dbapi)
        def selected(rows):
            self.assertEquals(len(rows),1)
            row=rows[0]
            self.assertEquals(str(type(row)),"<class 'tada.tao.TableRow'>")
            self.assertEquals(len(row.__all__()),len(simple.__all__()))
            self.assertEquals(row.num.table,simple)
            self.assertEquals(row.num.length,simple.num.length)
            self.assertEquals(row.num.null,simple.num.null)
            self.assertEquals(row.num.default,simple.num.default)
            self.assertEquals(row.num.value,1)
            return True
        def inserted(cursor):
            cursor.__tada_connection__.__tada_connection__.commit()
            return simple.select().addCallback(selected)
        def created(_):
            return dbapi.execute("INSERT INTO simple (num) VALUES (1)").addCallback(lambda cursor:inserted(cursor))
        return simple.create().addCallback(created)

    def test_Insert(self):
        filename = self.mktemp()
        simple = SimpleTao(Tadah("pysqlite2.dbapi2",filename))
        def inserted7(lastRowId):
            self.assertEquals(lastRowId,2)
            return True
        def inserted(lastRowId):
            self.assertEquals(lastRowId,1)
            return simple.insert(num=7).addCallback(inserted7)
        def created(cursor):
            return simple.insert(num=1).addCallback(inserted)
        return simple.create().addCallback(created)

